SystemOrganization addCategory: #'Pier-Exporter-Code'!

!Object methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/12/2011 16:58'!
asCode
	^ String streamContents: [ :stream | self codeOn: stream ]! !

!Object methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 17:09'!
codeOn: aStream
	self printOn: aStream! !

MAVisitor subclass: #PRCodeBuilderDescriptionVisitor
	instanceVariableNames: 'structure stream methods exporter selectorsWithCommonDefaults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Exporter-Code'!

PRCodeBuilderDescriptionVisitor class
	instanceVariableNames: 'aComponent'!

PRCodeBuilderDescriptionVisitor class
	instanceVariableNames: 'aComponent'!

!PRCodeBuilderDescriptionVisitor class methodsFor: 'instance creation' stamp: 'NickAger 1/26/2012 17:13'!
structure: structure methods: methods exporter: exporter on: stream
	^ self basicNew
		initializeWithStructure: structure methods: methods exporter: exporter stream: stream
		yourself! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'private' stamp: 'NickAger 1/26/2012 16:10'!
addCodeFor: aDescription value: value
	aDescription accessor asCodeFor: structure description: aDescription value: value on: stream.
	stream nextPut: Character cr! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'modifying' stamp: 'NickAger 12/12/2011 12:29'!
addMethod: method
	self methods add: method! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'initialization' stamp: 'NickAger 1/26/2012 17:14'!
initializeWithStructure: aStructure methods: methodsCollection exporter: anExporter stream: aStream
	self initialize.
	
	structure := aStructure.
	methods := methodsCollection.
	exporter := anExporter.
	stream := aStream! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'check defaults' stamp: 'NickAger 1/26/2012 15:56'!
isCommonDefault: aDescription
	| accessor descriptionDefault structureValue defaultCheckBlock |
	
	"Unfortunately the description frequently doesn't define a default value, but there are a common set of defaults we don't need to write.
	We key off the readSelector and check against a dictionary for common defaults"
	accessor := aDescription accessor.
	(accessor class canUnderstand: #readSelector) ifFalse: [ ^ false ].
	defaultCheckBlock := self selectorsWithCommonDefaults at: accessor readSelector ifAbsent: [ ^ false ].
	
	^ defaultCheckBlock value! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'check defaults' stamp: 'NickAger 1/26/2012 15:46'!
isDefaultDescriptionValue: aDescription
	| descriptionDefault structureValue |
	
	descriptionDefault := aDescription default.
	structureValue := aDescription accessor read: structure.
	^ descriptionDefault = structureValue ! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'check defaults' stamp: 'NickAger 1/26/2012 16:36'!
isNil: aDescription
	^ (aDescription accessor read: structure) = nil
! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'accessing' stamp: 'NickAger 12/12/2011 16:19'!
methods
	^ methods! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'check defaults' stamp: 'NickAger 1/26/2012 15:55'!
selectorsWithCommonDefaults
	^ selectorsWithCommonDefaults ifNil: [ 
		selectorsWithCommonDefaults := Dictionary new
			at: #name put: [ true ];
			at: #title put: [ structure title = structure name capitalized ];
			at: #menu put: [ structure menu = structure title ];
			at: #tags put: [ structure tags isEmpty ];
			at: #hideFromMenus put: [ structure hideFromMenus not ];
			yourself ]! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'visiting' stamp: 'NickAger 1/26/2012 16:34'!
visit: aDescription
	"only visit writable descriptions - otherwise we can't initialise it."
	(aDescription accessor canWrite: structure) ifFalse: [ ^ self ]. 
	
	"only visit descriptions with non-default values"
	(self isDefaultDescriptionValue: aDescription) ifTrue: [ ^ self ].
	(self isNil: aDescription) ifTrue: [ ^ self ].
	(self isCommonDefault: aDescription) ifTrue: [ ^ self ].

	aDescription acceptMagritte: self! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'visiting' stamp: 'NickAger 12/12/2011 15:16'!
visitDescription: aDescription
	| value |
	value := (aDescription accessor read: structure) asCode.
	self addCodeFor: aDescription value: value! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'visiting' stamp: 'NickAger 12/12/2011 15:15'!
visitDocumentDescription: aDescription
	"could have used:
		 value := (aDescription toString: (aDescription accessor read: structure)) asCode
	but I thought it was good to be explicit that we require wiki format output"
	| value |
	value := (PRWikiWriter write: (aDescription accessor read: structure)) asCode.
	self addCodeFor: aDescription value: value! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'visiting' stamp: 'NickAger 1/26/2012 16:11'!
visitFileDescription: aDescription
	| fileModel |
	fileModel := aDescription accessor read: structure.
	stream 
		<< '	structure file: (' << fileModel class greaseString << ' new
		filename: ' << fileModel filename asCode << ';
		mimetype: ' << fileModel mimetype asCode << ';
		contents: ' << (fileModel isText ifTrue: [ fileModel contents asString asCode ] ifFalse: [ fileModel contents greaseString ]) << ').' << String cr

! !

!PRCodeBuilderDescriptionVisitor methodsFor: 'visiting' stamp: 'NickAger 1/28/2012 19:09'!
visitStructureDescription: aDescription
	| structureSelector subStructure |
	subStructure := aDescription accessor read: structure.
	aDescription accessor asCodeFor: structure description: aDescription value: ' (self ', subStructure asSelector, ': kernel)' on: stream.
	stream nextPut: Character cr.
	
	self methods addAll: (exporter writeCodeFor: subStructure)! !

!WideString methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 19:09'!
codeOn: aStream
	aStream << '(WideString fromByteArray: '; print: self asByteArray; << ')'! !

!String methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 17:05'!
codeOn: aStream
	aStream nextPut: $'.
	self do: [ :aChar |
		aChar = $' ifTrue: [ aStream nextPut: $' ].
		aStream nextPut: aChar ].
	aStream nextPut:  $'! !

!MAAccessor methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 22:52'!
asCodeFor: structure description: description value: value on: stream
	| structureClass |
	structureClass := structure class greaseString.
	stream << '	' << structureClass << ' description detect: [ :each | each label = ''' << description label << ''' ]) accessor write: ' << value << ' to: structure. '! !

!PRDocumentAccessor methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 22:49'!
asCodeFor: structure description: description value: value on: stream
	stream << '	structure contents: ' << value << '.'! !

!Date methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 17:51'!
codeOn: aStream
	aStream << '(' << self class greaseString << ' year: '; print: self year; << ' month: '; print: self monthIndex; << ' day: '; print: self dayOfMonth; << ')'! !

!Duration methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/7/2012 17:16'!
codeOn: aStream
	aStream << '(' << self class greaseString <<  ' days: '; print: self days; << ' hours: '; print: self hours; << ' minutes: '; print: self minutes; << ' seconds: '; print: self seconds; << ')' ! !

!PRStructure methodsFor: '*pier-exporter-code' stamp: 'NickAger 1/28/2012 15:37'!
asSelector
	| aName selector |
	aName := self absolutePath.
	aName := (aName = '/') ifTrue: [ self name ] ifFalse: [ aName ].

	selector := aName select: [ :each | each isAlphaNumeric ].
	[ selector first isDigit ]
		whileTrue: [ selector := selector allButFirst ].
		
	selector at: 1 put: (selector at: 1) asLowercase.
		
	^ selector asSymbol! !

!PRStructure methodsFor: '*pier-exporter-code' stamp: 'NickAger 1/28/2012 19:06'!
codeOn: aStream
	aStream << '(self ' << self asSelector << ': kernel)'! !

!Symbol methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/12/2011 18:20'!
codeOn: aStream
	aStream << '#' << self! !

!MASelectorAccessor methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 22:52'!
asCodeFor: structure description: description value: value on: stream
	stream << '	structure ' << self writeSelector greaseString << ' ' << value << '.'! !

GRObject subclass: #PRKernelCodeExporter
	instanceVariableNames: 'selectorsWithCommonDefaults exportedMethods recursiveMethods prcomponentWriter add standardPermissions standardOtherPermissions isPier3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Exporter-Code'!

!PRKernelCodeExporter class methodsFor: 'instance creation' stamp: 'NickAger 2/7/2012 17:04'!
exportAsPier1Or2Kernel: kernelOrPierDispatchPath
	^ self new
		setOldPRComponentGeneration;
		exportWithPierKernelOrDispatchPath: kernelOrPierDispatchPath;
		yourself! !

!PRKernelCodeExporter class methodsFor: 'instance creation' stamp: 'NickAger 2/7/2012 17:04'!
exportAsPier3Kernel: kernelOrPierDispatchPath
	^ self new
		setPrototypePRComponentGeneration;
		exportWithPierKernelOrDispatchPath: kernelOrPierDispatchPath;
		yourself! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 19:02'!
addKernelCreatorMethod: aKernel forComponent: component
	| method |
	method := String streamContents: [ :stream |
		stream << 'createKernel' << String cr <<
			'	| kernel |' << String cr <<
			'	kernel := PRKernel named: ''' << aKernel name << '''.' << String cr <<
			'	self addGroupsToKernel: kernel.' << String cr <<
			'	self addUsersToKernel: kernel.' << String cr <<
			'	kernel root: (self rootPage: kernel).' ].
	self compileMethod: method intoComponent: component! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 18:51'!
addKernelGroups: kernel to: component
	| method |
	method := String streamContents: [ :stream |
		stream << 'addGroupsToKernel: kernel' << String cr <<
			'	kernel propertyAt: #groups put:' << String cr <<
			' 		(Set new' << String cr. 
		kernel groups do: [ :group |
			stream << '			add: '.
			self writeGroup: group on: stream ]. 
	stream << '			yourself).' << String cr.
	stream << '	kernel propertyAt: #adminGroup put: (kernel groupNamed: ''' << kernel adminGroup name << ''' ifNone: nil)' << String cr ].
	self compileMethod: method intoComponent: component! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 18:52'!
addKernelUsers: kernel to: component
	| method |
	method := String streamContents: [ :stream |
		stream << 'addUsersToKernel: kernel' << String cr <<
			'	kernel propertyAt: #users put: (Set new' << String cr. 
		kernel users do: [ :user |
			stream << '		add: '.
			self writeUser: user on: stream ]. 
	stream << '		yourself).' << String cr.
	stream << 
		'	kernel propertyAt: #adminUser put: (kernel userNamed: ''' << kernel adminUser name << ''' ifNone: nil)' << String cr ].
	self compileMethod: method intoComponent: component
	
! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 13:50'!
addPermissionSetCreationMethodOn: component
	| method |
	method := String streamContents: [ :stream |
		stream << 'permissionSetFrom: commands' << String cr <<
			'	^ (commands collect: [ :each | PUPermission on: each ]) asSet' ].
	self compileMethod: method intoComponent: component! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/26/2012 14:48'!
compile: aString into: aClass classified: classificationSymbol
	GRPlatform current 
		compile: aString
		into: aClass
		classified: classificationSymbol
! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/26/2012 14:49'!
compileMethod: method intoComponent: component
	self compile: method into: component classified: #'exported code'



! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 12/12/2011 20:12'!
compileMethods: methods intoComponent: component
	methods do: [ :method |
		self compileMethod: method intoComponent: component ]


! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 12/12/2011 21:28'!
createClassFor: kernel 
	^ self createClassFor: kernel instanceVariableNames: ''! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/26/2012 14:46'!
createClassFor: kernel instanceVariableNames: instanceVariableNames
	^ Object
		subclass: (self selectorFromName: kernel name, 'KernelCreator') capitalized  
		instanceVariableNames: instanceVariableNames
		classVariableNames: 'rootPage'
		poolDictionaries: ''
		category: 'Pier-Exported-Code'! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 2/7/2012 16:57'!
exportKernel: kernel
	| component rootComponent |
	
	component := self createClassFor: kernel.
	self addKernelCreatorMethod: kernel forComponent: component.
	self addKernelGroups: kernel to: component.
	self addKernelUsers: kernel to: component.
	self addPermissionSetCreationMethodOn: component.
	
	rootComponent := kernel root.
	
	self compile: (self rootPageTemplate: rootComponent)
		into: component
		classified: #'exported code'.
		
	rootComponent enumerator with all do: [ :structure |
		| methods |
		methods := self writeCodeFor: structure.
		self compileMethods: methods intoComponent: component ].
	
	self rewriteRecursiveMethodsFor: kernel! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 2/7/2012 17:04'!
exportWithPierKernelOrDispatchPath: kernelOrPierDispatchPath
	| kernel |
	
	kernel := kernelOrPierDispatchPath isString 
		ifTrue: [ ((WADispatcher default handlerAt: kernelOrPierDispatchPath) configuration valueAt: #kernel) ]
		ifFalse: [ kernelOrPierDispatchPath ].
	
	self exportKernel: kernel! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 2/7/2012 16:39'!
handlePier2PRComponentClassInitialisationFor: aPRComponent on: stream
	aPRComponent settings associationsDo: [ :aDescriptionValuePair |
		stream << '	structure write: ' << aDescriptionValuePair value asCode << 
			' using: ' << aPRComponent componentClass name << ' ' 
				<< (self selectorFromDescription: aDescriptionValuePair key onClass: aPRComponent componentClass)
				<< '.' << String cr.
		
		"if the value is a structure we need to make sure we avoid a recursive loop "
		(aDescriptionValuePair value isKindOf: PRStructure) ifTrue: [ 
			recursiveMethods add: aDescriptionValuePair value asSelector ] ]! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 2/7/2012 16:39'!
handlePier2PRComponentPrototypeInitialisationFor: aPRComponent on: stream
	stream << '	structure prototypeInstance: (' << aPRComponent componentClass name << ' new' << String cr.
	aPRComponent settings associationsDo: [ :aDescriptionValuePair |
		| accessor |
		accessor := aDescriptionValuePair key accessor.
		(accessor class canUnderstand: #readSelector) ifTrue: [
			stream << '		' << accessor readSelector << ': ' << aDescriptionValuePair value asCode << ';' << String cr ].
		
		"if the value is a structure we need to make sure we avoid a recursive loop "
		(aDescriptionValuePair value isKindOf: PRStructure) ifTrue: [ 
			recursiveMethods add: aDescriptionValuePair value asSelector ] ].
	stream << '		yourself).' << String cr.! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 2/7/2012 17:29'!
handlePier3PRComponentPrototypeInitialisationFor: aPRComponent on: stream
	| defaultValueComponent prototypeInstance |
	
	stream << '	structure prototypeInstance: (' << aPRComponent componentClass name << ' new' << String cr.
	
	defaultValueComponent := aPRComponent componentClass new.
	prototypeInstance := aPRComponent prototypeInstance.
	
	prototypeInstance magritteDescription do: [ :aDescription |
		| value |
		
		value := prototypeInstance readUsing: aDescription.
		"only write values which are different from the defaults"
		(value ~= (defaultValueComponent readUsing: aDescription)) ifTrue: [
			| accessor |
			accessor := aDescription accessor.
			(accessor class canUnderstand: #readSelector) ifTrue: [
				stream << '		' << accessor readSelector << ': ' << value asCode << ';' << String cr ].
		
			"if the value is a structure we need to make sure we avoid a recursive loop "
			(value isKindOf: PRStructure) ifTrue: [ 
				recursiveMethods add: value asSelector ] ] ].
	
	stream << '		yourself).' << String cr.	! !

!PRKernelCodeExporter methodsFor: 'initialization' stamp: 'NickAger 2/7/2012 17:03'!
initialize
 	super initialize.
	
	exportedMethods := Set new.
	recursiveMethods := Set new! !

!PRKernelCodeExporter methodsFor: 'helpers' stamp: 'NickAger 2/7/2012 15:08'!
isPier3
	^ isPier3 ifNil: [
		isPier3 := (GRPlatform current  magritteClassNamed: #MAPragmaBuilder) notNil ]! !

!PRKernelCodeExporter methodsFor: 'helpers' stamp: 'NickAger 2/7/2012 15:10'!
magritteSelector
	^ self isPier3  
		ifTrue: [ #magritteDescription ]
		ifFalse: [ #description ]! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 18:44'!
rewriteRecursiveMethodsFor: kernel
	| instanceVariables component |
	instanceVariables := String streamContents: [ :stream |
		recursiveMethods do: [ :each | 
			stream nextPutAll: each ]
			separatedBy: [ stream nextPut: Character space ] ].
		
	component := self createClassFor: kernel instanceVariableNames: instanceVariables.
	
	"this is somewhat hacky - we rewrite reentrant component creation methods with lazy initialiser using a regex match
	a two pass generator would remove the need for this hack - but would be more complex. "
	recursiveMethods do: [ :each | 
		| regex initialAssignment propertySetting method |
		regex := '.*(structure \:=[[:print:]]+)(.*)\^ structure' asRegex.
		regex search: (component sourceCodeAt: (each, ':') asSymbol).
		self assert: (regex subexpressionCount = 3).
		initialAssignment :=  regex subexpression: 2.
		propertySetting :=  regex subexpression: 3.
		method := String streamContents: [ :stream |
			stream 
				<<  each << ': kernel' << String cr
				<< '	 ^ ' << each << ' ifNil: [ ' << String cr
				<< '		| structure |' << String cr
				<< '		' << initialAssignment << String cr
				<< '		' << each << ' := structure.'
				<< propertySetting << String cr		
				<< '		' << each << ' ]' ].
				
		self compileMethod: method intoComponent: component ]! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 19:03'!
rootPageTemplate: rootComponent
	^ String streamContents: [ :stream | 
		stream << 'rootPage: kernel
	^ rootPage ifNil: [
		rootPage := self ' << (rootComponent asSelector) << ': kernel ]']! !

!PRKernelCodeExporter methodsFor: 'helpers' stamp: 'NickAger 1/26/2012 17:49'!
selectorFromDescription: aDescription onClass: aClass
	| descriptionSelectors |
	descriptionSelectors := SortedCollection withAll: (aClass class allSelectors select: [ :each | each isDescriptionDefinition ]).		
	
	^ descriptionSelectors detect: [ :each |  (aClass perform: each) = aDescription ]
	! !

!PRKernelCodeExporter methodsFor: 'helpers' stamp: 'NickAger 12/12/2011 20:13'!
selectorFromName: aName
	| selector |
	selector := aName select: [ :each | each isAlphaNumeric ].
	[ selector first isDigit ]
		whileTrue: [ selector := selector allButFirst ].
		
	selector at: 1 put: (selector at: 1) asLowercase.
		
	^ selector asSymbol! !

!PRKernelCodeExporter methodsFor: 'initialization' stamp: 'NickAger 2/7/2012 17:03'!
setOldPRComponentGeneration
	prcomponentWriter := [:aPRComponent :stream | self handlePRComponentClassInitialisationFor: aPRComponent on: stream ]! !

!PRKernelCodeExporter methodsFor: 'initialization' stamp: 'NickAger 2/7/2012 15:14'!
setPrototypePRComponentGeneration
	prcomponentWriter := self isPier3 
		ifTrue: [ [ :aPRComponent :stream | self handlePier3PRComponentPrototypeInitialisationFor: aPRComponent on: stream ] ]
		ifFalse: [  [ :aPRComponent :stream | self handlePier2PRComponentPrototypeInitialisationFor: aPRComponent on: stream ] ]! !

!PRKernelCodeExporter methodsFor: 'accessing' stamp: 'NickAger 1/29/2012 11:30'!
standardOtherPermissions
	^ standardOtherPermissions ifNil: [
		standardOtherPermissions := PUSecurity new otherPermissions ]! !

!PRKernelCodeExporter methodsFor: 'accessing' stamp: 'NickAger 1/29/2012 11:30'!
standardPermissions
	^ standardPermissions ifNil: [
		standardPermissions :=  PUSecurity new ownerPermissions ]! !

!PRKernelCodeExporter methodsFor: 'helpers' stamp: 'NickAger 2/7/2012 14:05'!
writeCodeFor: structure
	| methods method codeBuilder selector |
	selector := structure asSelector.
	
	"recursion guard"
	(exportedMethods includes: selector)
		ifTrue: [ recursiveMethods add: selector. ^ #() ]
		ifFalse: [ exportedMethods add: selector ].
	
	methods := OrderedCollection new.
	
	method :=  String streamContents: [ :stream | 
		stream << selector << ': kernel' << String cr << 
			'	| structure  |' << String cr <<
			'	structure := ' << structure class greaseString << ' named: ''' << structure name << '''.' << String cr.
	
		codeBuilder := PRCodeBuilderDescriptionVisitor structure: structure methods: methods exporter: self on: stream.
		(structure perform: self magritteSelector) do: [ :aDescription |
			codeBuilder visit: aDescription ].
		
		structure class name = #PRComponent ifTrue: [ prcomponentWriter value: structure value: stream  ].
			
		structure enumerator do: [ :each | 
			stream << '	structure addChild: (self ' << (each asSelector) << ': kernel).' << String cr ].
		
		stream nextPut: Character cr.	
		stream << '	structure securityDecoration owner: (kernel userNamed: ''' << structure securityDecoration owner name << ''' ifNone: nil).' << String cr.
		structure securityDecoration ownerPermissions = self standardPermissions ifFalse: [
			stream << '	structure securityDecoration ownerPermissions: '.
			self writePermissions: structure securityDecoration ownerPermissions on: stream ]. 
		
		stream << '	structure securityDecoration group: (kernel groupNamed: ''' << structure securityDecoration group name << ''' ifNone: nil).' << String cr.
		structure securityDecoration groupPermissions = self standardPermissions ifFalse: [
			stream << '	structure securityDecoration groupPermissions: '.
			self writePermissions: structure securityDecoration groupPermissions on: stream ]. 

		structure securityDecoration otherPermissions = self standardOtherPermissions ifFalse: [		
			stream << '	structure securityDecoration otherPermissions: '.
			self writePermissions: structure securityDecoration otherPermissions on: stream ]. 
		
		stream <<'	^ structure' ].
	
	methods add: method.
	^ methods! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 18:49'!
writeGroup: group on: stream
	stream << '(PUGroup named: ''' << group name << ''');' << String cr! !

!PRKernelCodeExporter methodsFor: 'helpers' stamp: 'NickAger 1/29/2012 11:15'!
writePermissions: permissions on: stream
	stream << '(self permissionSetFrom: {'.
	permissions do: [ :each |
		stream << each command name ] separatedBy: [ stream nextPutAll: '. '].
	stream << '}).' << String cr.! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 1/28/2012 19:01'!
writeUser: user on: stream
	stream << '((PUUser named: ''' << user name << ''')' << String cr <<
		'			groups: (OrderedCollection new' << String cr. 
	user groups do: [ :each |
		stream << '				add: (kernel groupNamed: ''' << each name << ''' ifNone: nil);' << String cr ].
	stream << '				yourself);' << String cr.
	stream << '			instVarNamed: ''password'' put: ' << (user instVarNamed: 'password') << ';' << String cr.
	stream << '			email: '; print: user email; nextPut: $;; nextPut: Character cr.
	stream << '			firstName: '; print: user firstName; nextPut: $;; nextPut: Character cr.
	stream << '			lastName: '; print: user lastName; nextPut: $;; nextPut: Character cr.			
	user superuser ifTrue: [
		stream << '			superuser: true;' << String cr ].
	stream << '			yourself);' << String cr! !

!UndefinedObject methodsFor: '*pier-exporter-code' stamp: 'NickAger 1/26/2012 16:18'!
codeOn: aStream
	aStream << 'nil'! !

!DateAndTime methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 17:55'!
codeOn: aStream
	aStream << '(' << self class greaseString << ' year: '; print: self year; << ' month: '; print: self monthIndex; << ' day: '; print: self dayOfMonth; << ' hour: '; print: self hour; << ' minute: '; print: self minute; << ' second: '; print: self second; << ')' ! !

!Collection methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 17:47'!
codeOn: aStream
	aStream << '#('.
	self do: [ :each | each codeOn: aStream ] separatedBy: [ aStream nextPutAll: ' '].
	aStream << ')'! !

!Array methodsFor: '*pier-exporter-code' stamp: 'NickAger 2/6/2012 17:47'!
codeOn: aStream
	aStream nextPut: ${.
	self do: [:el | el storeOn: aStream] separatedBy: [ aStream nextPutAll: '. '].
	aStream nextPut: $}! !
